/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package mechacrawler;

import java.net.*;
import java.util.*;
import java.util.Formatter;

/**
 *
 * @author isaaclw
 */
public class MechStore {

    // connection of links
    // <URL, Set of URLS>
    // Lets us find all the URLS that are pointing to a specific url
    // We used a Hash Map because we're just mapping the URLS to the set
    // Hash because it's faster
    private HashMap<URL, UrlSet> uset;    
    
    // < the word we're searching for, HashMap<URLS, URL to Int collections>
    // Lets us specify a word and find the links and occurances on a page with that link
    // We used a TreeMap because we wanted to sort the words in order
           // the hash map again just maps URLS to number-urls
    private TreeMap<String, HashMap<URL, TheWord>> wset;  
    
    // < URL, URL to Int collections> 
    // Lets us find a number of  occurances of a specific url
    // We used a Hash Map again, for the same resons.
    private HashMap<URL, TheWord> urls;
    private static int totSites;

    /**
     * Builds our collection classes
     */
    public MechStore() {
        uset = new HashMap<URL, UrlSet>();
        wset = new TreeMap<String, HashMap<URL, TheWord>>();
        totSites = 0;
        urls = new HashMap<URL, TheWord>();
    }

    /**
     * Associates two URLs together so that uset will list all the URLS that point to a URL
     * @param current the URL that we are at when we call this method
     * @param found the URL that we found when we call this method
     * @throws java.net.MalformedURLException
     * @throws java.lang.InterruptedException
     */
    public void assosiate(URL current, URL found) throws MalformedURLException, InterruptedException {
        // takes care of links between urls
        if (uset.containsKey(found)) {
            uset.get(found).add(current);
        } else {
            uset.put(current, new UrlSet(found));
        }
        // keeps track of the number of occurances of a url
        if (urls.containsKey(found)) {
            urls.get(found).found();
        } else {
            urls.put(found, new TheWord(found));
            Crawler.add(found); // we only want to run it if it didn't show up yet.

            totSites++;
            MechaWar.addTotal(totSites);
        }
    }

    /**
     * Adds a word to our 'wset'
     * @param word the Word that we just found
     * @param current the current URL that we found the word on
     */
    public void addWord(String word, URL current) {
        if (wset.containsKey(word)) {
            if (wset.get(word).containsKey(current)) {
                wset.get(word).get(current).found();
            } else { // the url doesn't exist yet
                wset.get(word).put(current, new TheWord(current));
            }
        } else { // the word doesn't exist yet
            wset.put(word, new HashMap<URL, TheWord>());
            wset.get(word).put(current, new TheWord(current));
        }
    //System.out.println("The word: " + word + " was added");
    }

    /**
     * Sends "MechaWar" a un-sorted list of
     * [number] [link]
     * where [number] is the number of occurances of [link]
     */
    public synchronized void displayUrls() {
        
        for (URL url : urls.keySet()) {
            MechaWar.appendCraDisp(urls.get(url).number() + " " + url);
        }
    }

    /**
     * Sends "MechaWar" a sorted list of
     * [word] [number] [link]
     * where [number] is the number of occurances of [link]
     */
    public synchronized void displayWords() {

        for (String word : wset.keySet()) {
            MechaWar.appendCraDisp(word);
            for (TheWord url : wset.get(word).values()) {
                MechaWar.appendCraDisp("\t" + url.number() + "   " + url.toString());
            }
        }

    }

    /**
     * Sends "MechaWar" a sorted list of
     * [number] [link]
     * where [number] is the number of words on the page [link]
     * @param word the word to search for
     */
    public synchronized void occurance(String word) {

        if (wset.containsKey(word)) {
            TreeSet<String> list = new TreeSet<String>();

            for (URL url : wset.get(word).keySet()) {
                String add = String.format(" %4d %s", wset.get(word).get(url).number(), url);
                list.add(add);
            }
            for (String i : list) {
                MechaWar.appendSeaDisp(i);
            }
        } else {
            MechaWar.appendSeaDisp("Word not Found");
        }
    }

    /**
     * Sends MechaWar a sorted list of
     * [number] [link]
     * where [number] is the number of links that point to [link]
     * @param word the word to search for
     */
    public synchronized void supportLinks(String word) {

        if (wset.containsKey(word)) {
            if (wset.get(word).size() < 45) {
            TreeSet<String> list = new TreeSet<String>();
            for (URL url : wset.get(word).keySet()) {
                list.add(String.format("%4d %s", uset.get(url).size(), url));
            }
            for (String i : list) {
                MechaWar.appendSeaDisp(i);
            }
            }
            else {
            MechaWar.appendSeaDisp("List too Large. Try searching for a less frequent word");
                
            }
            
        } else {
            MechaWar.appendSeaDisp("Word not Found");
        }
    }
    // Sort by word occurance
    // Sort by links to the page

    /**
     * a class that stores a set of URLS
     */
    class UrlSet {
        private HashSet<URL> collection;
        /**
         * contructing the set
         * @param aURL a starting url
         */
        UrlSet(URL aURL) {
            collection = new HashSet<URL>();
            collection.add(aURL);
        }
        void add(URL aURL) {
            collection.add(aURL);
        }
        public int size() {
            return collection.size();
        }
    }

    /**
     * a class to store a number with a URL
     */
    class TheWord {

        private int n;  // a number to a URL
        private URL ourURL;
        /**
         * Stores the URL
         * @param aURL
         */
        TheWord(URL aURL) {
            n = 1;
            ourURL = aURL;
        }
        public String toString() {
            return "" + ourURL;
        }
        /**
         * @return the number to that url
         */
        public int number() {
            return n;
        }
        /**
         * increments the number to that URL
         */
        public void found() {
            n++;
        }
        /**
         * checks if the urls are the same
         * @param url the url to pass in
         * @return true if they're the same
         */
        public boolean compare(URL url) {
            return (ourURL.equals(url));
        }
    }
}
